@@ -6,7 +6,7 @@ import libsbml
 
 def write_ids_to_names(input_path: Path, output_path: Path) -> None:
     """Write SBML ids as names."""
-    doc = libsbml.readSBMLFromFile(str(input_path))  # type: libsbml.SBMLDocument
+    doc: libsbml.SBMLDocument = libsbml.readSBMLFromFile(str(input_path))
     elements = doc.getListOfAllElements()
     for element in elements:
         if element.isSetId():
