@@ -7,11 +7,12 @@ In build in python are
     and, or, not
 """
 from math import *
+from typing import Any, Dict, Optional, Tuple
 
 import libsbml
 
 
-def product(*args):
+def product(*args: float) -> float:
     """Product calculation."""
     res = 1.0
     for arg in args:
@@ -19,17 +20,17 @@ def product(*args):
     return res
 
 
-def sqr(x):
+def sqr(x: float) -> float:
     """Square calculation."""
     return x * x
 
 
-def root(a, b):
+def root(a: float, b: float) -> float:
     """Root calculation."""
     return a ** (1 / b)
 
 
-def xor(*args):
+def xor(*args: float) -> int:
     """XOR calculation."""
     foundZero = 0
     foundOne = 0
@@ -44,7 +45,7 @@ def xor(*args):
         return 0
 
 
-def piecewise(*args):
+def piecewise(*args: float) -> float:
     """Piecewise calculation."""
     Nargs = len(args)
     for k in range(0, Nargs - 1, 2):
@@ -116,21 +117,19 @@ def f_or(*args):
 """
 
 
-def evaluableMathML(astnode, variables=None, array=False):
-    """Create evaluable python string."""
+def evaluableMathML(astnode: libsbml.ASTNode, variables: Optional[Dict] = None) -> str:
+    """Create evaluable python formula string from ASTNode."""
     if variables is None:
         variables = {}
     # replace variables with provided values
     for key, value in variables.items():
         astnode.replaceArgument(key, libsbml.parseFormula(str(value)))
 
-    # get formula
-    # formula = libsbml.formulaToL3String(astnode)
-    settings = libsbml.L3ParserSettings()  # type: libsbml.L3ParserSettings
+    # parse formula
+    settings: libsbml.L3ParserSettings = libsbml.L3ParserSettings()
     settings.setParseUnits(False)
     settings.setParseCollapseMinus(True)
-
-    formula = libsbml.formulaToL3StringWithSettings(astnode, settings)
+    formula: str = libsbml.formulaToL3StringWithSettings(astnode, settings)
 
     # <replacements>
     formula = formula.replace("&&", "and")
@@ -140,21 +139,16 @@ def evaluableMathML(astnode, variables=None, array=False):
     return formula
 
 
-def evaluateMathML(astnode, variables=None, array=False):
+def evaluateMathML(astnode: libsbml.ASTNode, variables: Optional[Dict] = None) -> Any:
     """Evaluate MathML string with given set of variable and parameter values.
 
     :param astnode: astnode of MathML string
-    :type astnode: libsbml.ASTNode
     :param variables: dictionary of var : value
-    :type variables: dict
-    :param parameters: dictionary of par : value
-    :type parameters: dict
     :return: value of evaluated MathML
-    :rtype: float
     """
     if variables is None:
         variables = {}
-    formula = evaluableMathML(astnode, variables=variables, array=array)
+    formula = evaluableMathML(astnode, variables=variables)
     print(formula)
     # return the evaluated formula
     return eval(formula)
