@@ -11,21 +11,20 @@ from sbmlutils.validation import check
 # TODO: read the example cases
 
 
-def _distrib_doc():
+def _distrib_doc() -> libsbml.SBMLDocument:
     """Create distrib document."""
     sbml_level = 3
     sbml_version = 1
     sbmlns = libsbml.SBMLNamespaces(sbml_level, sbml_version)
     sbmlns.addPackageNamespace("distrib", 1)
-    doc = libsbml.SBMLDocument(sbmlns)  # type: libsbml.SBMLDocument
+    doc: libsbml.SBMLDocument = libsbml.SBMLDocument(sbmlns)
     doc.setPackageRequired("distrib", True)
 
     return doc
 
 
-def _create_parameter(pid, model: libsbml.Model):
-    # parameter
-    p = model.createParameter()  # type: libsbml.Parameter
+def _create_parameter(pid: str, model: libsbml.Model) -> libsbml.Parameter:
+    p: libsbml.Parameter = model.createParameter()
     p.setId(pid)
     p.setValue(1.0)
     p.setConstant(False)
@@ -34,19 +33,19 @@ def _create_parameter(pid, model: libsbml.Model):
     return p
 
 
-def distrib_normal():
+def distrib_normal() -> libsbml.SBMLDocument:
     """Create simple distrib model.
 
     :return:
     """
-    doc = _distrib_doc()
-    model = doc.createModel()  # type: libsbml.Model
+    doc: libsbml.SBMLDocument = _distrib_doc()
+    model: libsbml.Model = doc.createModel()
 
     # parameter
-    _ = _create_parameter("p1", model=model)  # type: libsbml.Parameter
+    _: libsbml.Parameter = _create_parameter("p1", model=model)
 
     # initial assignment
-    assignment = model.createInitialAssignment()  # type: libsbml.InitialAssignment
+    assignment: libsbml.InitialAssignment = model.createInitialAssignment()
     assignment.setSymbol("p1")
     ast_node = libsbml.parseL3FormulaWithModel("1.0 mole * normal(0, 1.0)", model)
     assignment.setMath(ast_node)
@@ -54,13 +53,13 @@ def distrib_normal():
     return doc
 
 
-def distrib_all():
+def distrib_all() -> libsbml.SBMLDocument:
     """Create simple distrib model.
 
     :return:
     """
-    doc = _distrib_doc()
-    model = doc.createModel()  # type: libsbml.Model
+    doc: libsbml.SBMLDocument = _distrib_doc()
+    model: libsbml.Model = doc.createModel()
 
     # extended math approach
     formulas_data = [
@@ -90,44 +89,41 @@ def distrib_all():
 
     # create parameters with distribution assignments
     for pid, formula in formulas_data:
-        print("{} = {}".format(pid, formula))
-        _ = _create_parameter(pid, model=model)  # type: libsbml.Parameter
-        assignment = model.createInitialAssignment()  # type: libsbml.InitialAssignment
+        # print(f"{pid} = {formula}")
+        _: libsbml.Parameter = _create_parameter(pid, model=model)
+        assignment: libsbml.InitialAssignment = model.createInitialAssignment()
         assignment.setSymbol(pid)
         ast_node = libsbml.parseL3FormulaWithModel(formula, model)
         if ast_node is None:
-            raise IOError("{}, {}".format(formula, libsbml.getLastParseL3Error()))
+            raise IOError(f"{formula}, {libsbml.getLastParseL3Error()}")
         assignment.setMath(ast_node), "setting math"
 
     return doc
 
 
-def uncertainty():
-    """Create uncertainty with UncertParameter.
-
-    :return:
-    """
-    doc = _distrib_doc()
-    model = doc.createModel()  # type: libsbml.Model
+def uncertainty() -> libsbml.SBMLDocument:
+    """Create uncertainty with UncertParameter."""
+    doc: libsbml.SBMLDocument = _distrib_doc()
+    model: libsbml.Model = doc.createModel()
 
     # parameter
-    p = _create_parameter("p1", model=model)  # type: libsbml.Parameter
-    p_distrib = p.getPlugin("distrib")  # type: libsbml.DistribSBasePlugin
+    p: libsbml.Parameter = _create_parameter("p1", model=model)
+    p_distrib: libsbml.DistribSBasePlugin = p.getPlugin("distrib")
 
     # --------------------------------------------
     # Build generic uncertainty for parameter
     # --------------------------------------------
     # 5.0 (mean) +- 0.3 (std) [2.0 - 8.0]
 
-    uncertainty = p_distrib.createUncertainty()  # type: libsbml.Uncertainty
+    uncertainty: libsbml.Uncertainty = p_distrib.createUncertainty()
     uncertainty.setName("Basic example: 5.0 +- 0.3 [2.0 - 8.0]")
     unit = libsbml.UnitKind_toString(libsbml.UNIT_KIND_MOLE)
-    up_mean = uncertainty.createUncertParameter()  # type: libsbml.UncertParameter
+    up_mean: libsbml.UncertParameter = uncertainty.createUncertParameter()
     up_mean.setType(libsbml.DISTRIB_UNCERTTYPE_MEAN)
     up_mean.setValue(5.0)
     up_mean.setUnits(unit)
 
-    up_sd = uncertainty.createUncertParameter()  # type: libsbml.UncertParameter
+    up_sd: libsbml.UncertParameter = uncertainty.createUncertParameter()
     up_sd.setType(libsbml.DISTRIB_UNCERTTYPE_STANDARDDEVIATION)
     up_sd.setValue(0.3)
     up_sd.setUnits(unit)
@@ -168,7 +164,7 @@ def uncertainty():
     # Set of all UncertParameters
     # --------------------------------------------
     # create second uncertainty which contains all the individual uncertainties
-    uncertainty = p_distrib.createUncertainty()  # type: libsbml.Uncertainty
+    uncertainty = p_distrib.createUncertainty()
     uncertainty.setName("UncertParameter example")
     for k, parameter_type in enumerate(
         [
@@ -185,7 +181,7 @@ def uncertainty():
         ]
     ):
 
-        up = uncertainty.createUncertParameter()  # type: libsbml.UncertParameter
+        up: libsbml.UncertParameter = uncertainty.createUncertParameter()
         up.setType(parameter_type)
         up.setValue(k)
         up.setUnits(unit)
@@ -193,7 +189,7 @@ def uncertainty():
     # --------------------------------------------
     # Set of all UncertSpans
     # --------------------------------------------
-    uncertainty = p_distrib.createUncertainty()  # type: libsbml.Uncertainty
+    uncertainty = p_distrib.createUncertainty()
     uncertainty.setName("UncertSpan example")
     for k, parameter_type in enumerate(
         [
@@ -215,9 +211,9 @@ def uncertainty():
     # Use math for distribution definition
     # --------------------------------------------
     # 5.0 dimensionless * normal(1.0 mole, 3.0 mole)
-    uncertainty = p_distrib.createUncertainty()  # type: libsbml.Uncertainty
+    uncertainty = p_distrib.createUncertainty()
     uncertainty.setName("math example: 5.0 dimensionless * normal(1.0 mole, 3.0 mole)")
-    up = uncertainty.createUncertParameter()  # type: libsbml.UncertParameter
+    up = uncertainty.createUncertParameter()
     up.setType(libsbml.DISTRIB_UNCERTTYPE_DISTRIBUTION)
     up.setDefinitionURL("http://www.sbml.org/sbml/symbols/distrib/normal")
     ast = libsbml.parseL3FormulaWithModel(
@@ -231,9 +227,9 @@ def uncertainty():
     # Use externalParameter
     # --------------------------------------------
     # https://sites.google.com/site/probonto/
-    uncertainty = p_distrib.createUncertainty()  # type: libsbml.Uncertainty
+    uncertainty = p_distrib.createUncertainty()
     uncertainty.setName("ExternalParameter example")
-    up = uncertainty.createUncertParameter()  # type: libsbml.UncertParameter
+    up = uncertainty.createUncertParameter()
     up.setType(libsbml.DISTRIB_UNCERTTYPE_EXTERNALPARAMETER)
     up.setName("skewness")
     up.setValue(0.25)
@@ -243,16 +239,16 @@ def uncertainty():
     # --------------------------------------------
     # Use external distribution definition
     # --------------------------------------------
-    uncertainty = p_distrib.createUncertainty()  # type: libsbml.Uncertainty
+    uncertainty = p_distrib.createUncertainty()
     uncertainty.setName("External distribution example")
-    up = uncertainty.createUncertParameter()  # type: libsbml.UncertParameter
+    up = uncertainty.createUncertParameter()
     up.setType(libsbml.DISTRIB_UNCERTTYPE_DISTRIBUTION)
     up.setName("Geometric 1")
     up.setDefinitionURL("http://www.probonto.org/ontology#PROB_k0000782")
     up.setUnits(unit)
 
     # success probability of Geometric-1
-    up_mean_geo1 = up.createUncertParameter()  # type: libsbml.UncertParameter
+    up_mean_geo1 = up.createUncertParameter()
     up_mean_geo1.setType(libsbml.DISTRIB_UNCERTTYPE_EXTERNALPARAMETER)
     up_mean_geo1.setName("success probability of Geometric 1")
     up_mean_geo1.setValue(0.4)
@@ -261,7 +257,7 @@ def uncertainty():
     return doc
 
 
-def create_examples(tmp=False):
+def create_examples(tmp: bool = False) -> None:
     """Create distrib examples."""
     functions = [
         distrib_normal,
