@@ -1,17 +1,18 @@
 """cobrapy based helper methods."""
-import logging
 from pathlib import Path
 from typing import Dict
 
 import pandas as pd
 
+from sbmlutils import log
+
 
 try:
     import cobra
 except ImportError:
     cobra = None
 
-logger = logging.getLogger(__name__)
+logger = log.get_logger(__name__)
 
 
 def read_cobra_model(sbml_path: Path) -> "cobra.core.Model":
@@ -26,7 +27,7 @@ def read_cobra_model(sbml_path: Path) -> "cobra.core.Model":
     return cobra.io.read_sbml_model(str(sbml_path))
 
 
-def cobra_reaction_info(cobra_model: "cobra.core.Model"):
+def cobra_reaction_info(cobra_model: "cobra.core.Model") -> pd.DataFrame:
     """Create data frame with bound and objective information.
 
     :param cobra_model:
