@@ -1,65 +1,69 @@
-from typing import Dict
-
 import libsbml
 
-from sbmlutils.creator import CoreModel
 from sbmlutils.distrib import distrib_examples, distrib_packages
 from sbmlutils.factory import *
 from sbmlutils.metadata import *
-from sbmlutils.units import *
 from sbmlutils.validation import validate_doc
 
 
-def test_distrib_examples():
+class U(Units):
+    """UnitsDefinition."""
+
+    hr = UnitDefinition("hr")
+    m2 = UnitDefinition("m2", "meter^2")
+    mM = UnitDefinition("mM", "mmole/liter")
+
+
+def test_distrib_examples() -> None:
     distrib_examples.create_examples(tmp=True)
 
 
-def test_add_uncertainty_example():
+def test_add_uncertainty_example() -> None:
     distrib_packages.add_uncertainty_example(tmp=True)
 
 
-def check_model_dict(d: Dict) -> libsbml.SBMLDocument:
-    """Check that no errors."""
+def check_model(model: Model) -> libsbml.SBMLDocument:
+    """Check that no errors in given model."""
     # create model and print SBML
-    core_model = CoreModel.from_dict(model_dict=d)
-    core_model.create_sbml()
-    assert core_model.doc is not None
-    vresults = validate_doc(core_model.doc, units_consistency=False)
+    doc: libsbml.SBMLDocument = Document(model=model).create_sbml()
+
+    assert doc
+    vresults = validate_doc(doc, units_consistency=False)
 
     # debugging
     if vresults.error_count > 0:
-        doc = core_model.doc  # type: libsbml.SBMLDocument
-        error_log = doc.getErrorLog()  # type: libsbml.SBMLErrorLog
+        error_log: libsbml.SBMLErrorLog = doc.getErrorLog()
         print(error_log.toString())
 
     assert vresults.is_valid()
-    return core_model.doc
+    return doc
 
 
-def test_assign_distribution():
+def test_assign_distribution() -> None:
     model_dict = {
-        "mid": "distrib_assignment",
+        "sid": "distrib_assignment",
         "packages": ["distrib"],
         "model_units": ModelUnits(
-            time=UNIT_hr,
-            extent=UNIT_KIND_MOLE,
-            substance=UNIT_KIND_MOLE,
-            length=UNIT_m,
-            area=UNIT_m2,
-            volume=UNIT_KIND_LITRE,
+            time=U.hr,
+            extent=U.mole,
+            substance=U.mole,
+            length=U.meter,
+            area=U.m2,
+            volume=U.liter,
         ),
-        "units": [UNIT_hr, UNIT_m, UNIT_m2, UNIT_mM],
-        "parameters": [Parameter(sid="p1", value=0.0, unit=UNIT_mM)],
+        "units": U,
+        "parameters": [Parameter(sid="p1", value=0.0, unit=U.mM)],
         "assignments": [
             InitialAssignment("p1", "normal(0 mM, 1 mM)"),
         ],
     }
-    check_model_dict(model_dict)
+    model: Model = Model(**model_dict)  # type: ignore
+    check_model(model)
 
 
-def test_normal_distribution():
+def test_normal_distribution() -> None:
     model_dict = {
-        "mid": "normal",
+        "sid": "normal",
         "packages": ["distrib"],
         "parameters": [
             Parameter("y", value=1.0),
@@ -69,12 +73,12 @@ def test_normal_distribution():
             InitialAssignment("y", "normal(z, 10)"),
         ],
     }
-    check_model_dict(model_dict)
+    check_model(Model(**model_dict))  # type: ignore
 
 
-def test_trunctated_normal_distribution():
+def test_trunctated_normal_distribution() -> None:
     model_dict = {
-        "mid": "truncated_normal",
+        "sid": "truncated_normal",
         "packages": ["distrib"],
         "parameters": [
             Parameter("y", value=1.0),
@@ -84,12 +88,12 @@ def test_trunctated_normal_distribution():
             InitialAssignment("y", "normal(z, 10, z-2, z+2)"),
         ],
     }
-    check_model_dict(model_dict)
+    check_model(Model(**model_dict))  # type: ignore
 
 
-def test_conditional_event():
+def test_conditional_event() -> None:
     model_dict = {
-        "mid": "conditional_events",
+        "sid": "conditional_events",
         "packages": ["distrib"],
         "parameters": [Parameter("x", value=1.0, constant=False)],
         "events": [
@@ -111,12 +115,12 @@ def test_conditional_event():
             ),
         ],
     }
-    check_model_dict(model_dict)
+    check_model(Model(**model_dict))  # type: ignore
 
 
-def test_overview_distributions():
+def test_overview_distributions() -> None:
     model_dict = {
-        "mid": "all_distributions",
+        "sid": "all_distributions",
         "packages": ["distrib"],
         "assignments": [
             InitialAssignment("p_normal_1", "normal(0, 1)"),
@@ -143,14 +147,14 @@ def test_overview_distributions():
             InitialAssignment("p_raleigh_2", "rayleigh(0.5, 0, 10)"),
         ],
     }
-    check_model_dict(model_dict)
+    check_model(Model(**model_dict))  # type: ignore
 
 
-def test_basic_uncertainty_example():
+def test_basic_uncertainty_example() -> None:
     import libsbml
 
     model_dict = {
-        "mid": "basic_example_1",
+        "sid": "basic_example_1",
         "packages": ["distrib"],
         "compartments": [Compartment("C", value=1.0)],
         "species": [
@@ -171,29 +175,27 @@ def test_basic_uncertainty_example():
             )
         ],
     }
-    check_model_dict(model_dict)
+    check_model(Model(**model_dict))  # type: ignore
 
 
-def test_multiple_uncertainties():
-    import libsbml
-
+def test_multiple_uncertainties() -> None:
     model_dict = {
-        "mid": "multiple_uncertainties",
+        "sid": "multiple_uncertainties",
         "packages": ["distrib"],
         "model_units": ModelUnits(
-            time=UNIT_hr,
-            extent=UNIT_KIND_MOLE,
-            substance=UNIT_KIND_MOLE,
-            length=UNIT_m,
-            area=UNIT_m2,
-            volume=UNIT_KIND_LITRE,
+            time=U.hr,
+            extent=U.mole,
+            substance=U.mole,
+            length=U.meter,
+            area=U.m2,
+            volume=U.liter,
         ),
-        "units": [UNIT_hr, UNIT_m, UNIT_m2, UNIT_mM],
+        "units": U,
         "parameters": [
             Parameter(
                 sid="p1",
                 value=5.0,
-                unit=UNIT_mM,
+                unit=U.mM,
                 uncertainties=[
                     Uncertainty(
                         "p1_uncertainty_1",
@@ -201,18 +203,20 @@ def test_multiple_uncertainties():
                             UncertParameter(
                                 type=libsbml.DISTRIB_UNCERTTYPE_MEAN,
                                 value=5.0,
-                                unit=UNIT_mM,
+                                unit=U.mM,
                             ),
                             UncertParameter(
                                 type=libsbml.DISTRIB_UNCERTTYPE_STANDARDDEVIATION,
                                 value=0.3,
-                                unit=UNIT_mM,
+                                unit=U.mM,
                             ),
+                        ],
+                        uncertSpans=[
                             UncertSpan(
                                 type=libsbml.DISTRIB_UNCERTTYPE_RANGE,
                                 valueLower=2.0,
                                 valueUpper=8.0,
-                                unit=UNIT_mM,
+                                unit=U.mM,
                             ),
                         ],
                     ),
@@ -222,18 +226,20 @@ def test_multiple_uncertainties():
                             UncertParameter(
                                 type=libsbml.DISTRIB_UNCERTTYPE_MEAN,
                                 value=4.5,
-                                unit=UNIT_mM,
+                                unit=U.mM,
                             ),
                             UncertParameter(
                                 type=libsbml.DISTRIB_UNCERTTYPE_STANDARDDEVIATION,
                                 value=1.1,
-                                unit=UNIT_mM,
+                                unit=U.mM,
                             ),
+                        ],
+                        uncertSpans=[
                             UncertSpan(
                                 type=libsbml.DISTRIB_UNCERTTYPE_RANGE,
                                 valueLower=1.0,
                                 valueUpper=10.0,
-                                unit=UNIT_mM,
+                                unit=U.mM,
                             ),
                         ],
                     ),
@@ -244,31 +250,30 @@ def test_multiple_uncertainties():
             InitialAssignment("p1", "normal(0 mM, 1 mM)"),
         ],
     }
-    doc = check_model_dict(model_dict)  # type: libsbml.SBMLDocument
+    doc: libsbml.SBMLDocument = check_model(Model(**model_dict))  # type: ignore
     assert doc
-    model = doc.getModel()  # type: libsbml.Model
+    model: libsbml.Model = doc.getModel()
     assert model
-    p = model.getParameter("p1")  # type: libsbml.Parameter
+    p: libsbml.Parameter = model.getParameter("p1")
     assert p
-    p_distrib = p.getPlugin("distrib")  # type: libsbml.DistribSBasePlugin
+    p_distrib: libsbml.DistribSBasePlugin = p.getPlugin("distrib")
     assert p_distrib
-    list_uncertainties = (
-        p_distrib.getListOfUncertainties()
-    )  # type: libsbml.ListOfUncertainties
+    list_uncertainties: libsbml.ListOfUncertainties = p_distrib.getListOfUncertainties()
+
     assert list_uncertainties
     n_uncertainties = p_distrib.getNumUncertainties()
     assert n_uncertainties == 2
     for k in range(n_uncertainties):
-        uc = p_distrib.getUncertainty(k)  # type: libsbml.Uncertainty
+        uc: libsbml.Uncertainty = p_distrib.getUncertainty(k)
         assert uc
         assert uc.isSetId()
 
 
-def test_define_random_variable():
+def test_define_random_variable() -> None:
     import libsbml
 
     model_dict = {
-        "mid": "random_variable",
+        "sid": "random_variable",
         "packages": ["distrib"],
         "parameters": [
             Parameter("shape_Z", value=10.0),
@@ -292,14 +297,12 @@ def test_define_random_variable():
             ),
         ],
     }
-    check_model_dict(model_dict)
+    check_model(Model(**model_dict))  # type: ignore
 
 
-def test_parameters_and_spans():
-    import libsbml
-
+def test_parameters_and_spans() -> None:
     model_dict = {
-        "mid": "parameters_spans",
+        "sid": "parameters_spans",
         "packages": ["distrib"],
         "parameters": [
             Parameter(
@@ -340,6 +343,8 @@ def test_parameters_and_spans():
                             UncertParameter(
                                 type=libsbml.DISTRIB_UNCERTTYPE_VARIANCE, value=10.0
                             ),
+                        ],
+                        uncertSpans=[
                             UncertSpan(
                                 type=libsbml.DISTRIB_UNCERTTYPE_CONFIDENCEINTERVAL,
                                 valueLower=1.0,
@@ -366,29 +371,29 @@ def test_parameters_and_spans():
             )
         ],
     }
-    check_model_dict(model_dict)
+    check_model(Model(**model_dict))  # type: ignore
 
 
-def test_sabiork_uncertainty():
+def test_sabiork_uncertainty() -> None:
     model_dict = {
-        "mid": "sabiork_parameter",
+        "sid": "sabiork_parameter",
         "packages": ["distrib"],
         "model_units": ModelUnits(
-            time=UNIT_hr,
-            extent=UNIT_KIND_MOLE,
-            substance=UNIT_KIND_MOLE,
-            length=UNIT_m,
-            area=UNIT_m2,
-            volume=UNIT_KIND_LITRE,
+            time=U.hr,
+            extent=U.mole,
+            substance=U.mole,
+            length=U.meter,
+            area=U.m2,
+            volume=U.liter,
         ),
-        "units": [UNIT_hr, UNIT_m, UNIT_m2, UNIT_mM],
+        "units": U,
         "parameters": [
             Parameter(
                 sid="Km_glc",
                 name="Michelis-Menten constant glucose",
                 value=5.0,
-                unit=UNIT_mM,
-                sboTerm=SBO_MICHAELIS_CONSTANT,
+                unit=U.mM,
+                sboTerm=SBO.MICHAELIS_CONSTANT,
                 uncertainties=[
                     Uncertainty(
                         sid="uncertainty1",
@@ -406,7 +411,7 @@ def test_sabiork_uncertainty():
                             (BQB.HAS_TAXON, "taxonomy/9606"),  # homo sapiens
                             (BQB.IS, "ec-code/2.7.1.2"),  # glucokinase
                             (BQB.IS, "uniprot/P35557"),  # Glucokinase homo sapiens
-                            (BQB.IS, "bto/BTO:000075"),  # liver
+                            (BQB.IS, "bto/BTO:0000075"),  # liver
                         ],
                     ),
                     Uncertainty(
@@ -426,11 +431,11 @@ def test_sabiork_uncertainty():
                             (BQB.HAS_TAXON, "taxonomy/9606"),  # homo sapiens
                             (BQB.IS, "ec-code/2.7.1.2"),  # glucokinase
                             (BQB.IS, "uniprot/P35557"),  # Glucokinase homo sapiens
-                            (BQB.IS, "bto/BTO:000075"),  # liver
+                            (BQB.IS, "bto/BTO:0000075"),  # liver
                         ],
                     ),
                 ],
             )
         ],
     }
-    check_model_dict(model_dict)
+    check_model(Model(**model_dict))  # type: ignore
